@@ -28,6 +28,7 @@
 
 #include <resParser.h>
 #include <exceptions.h>
+#include <util.h>
 #include <help.h>
 
 #ifndef INCL_RESPARSER_Y
@@ -279,6 +280,7 @@ StringToken accelList[] = {
    { "EditLeft", ACCEL_EDIT_LEFT, 0 },
    { "EditMiddle", ACCEL_EDIT_MIDDLE, 0 },
    { "EditRight", ACCEL_EDIT_RIGHT, 0 },
+   { "SaveOptions", ACCEL_SAVE_OPTIONS, 0 },
    { "Search", ACCEL_SEARCH, 0 },
    { "SearchForward", ACCEL_SEARCH_FORWARD, 0 },
    { "SearchBackward", ACCEL_SEARCH_BACKWARD, 0 },
@@ -361,9 +363,10 @@ StringToken accelList[] = {
    { "IgnoreFileRight", ACCEL_IGNORE_FILE_RIGHT, 0 },
    { "HelpManPage", ACCEL_HELP_MAN_PAGE, 0 },
    { "HelpOnContext", ACCEL_HELP_ON_CONTEXT, 0 },
-   { "HelpGenInitFile", ACCEL_HELP_GEN_INIT_FILE, 0 },
    { "HelpAbout", ACCEL_HELP_ABOUT, 0 },
-   { "MergedClose", ACCEL_MERGED_CLOSE, 0 }
+   { "MergedClose", ACCEL_MERGED_CLOSE, 0 },
+   { "FontResizeBigger", ACCEL_FONT_RESIZE_BIGGER, 0 },
+   { "FontResizeSmaller", ACCEL_FONT_RESIZE_SMALLER, 0 },
 };
 
 StringToken colorList[] = {
@@ -1215,12 +1218,12 @@ void XxResParser::genInitFile(
    if ( res1.getClipboardHeadFormat() !=
         res2.getClipboardHeadFormat() ) {
       os << searchTokenName( STPARAM(kwdList), CLIPBOARD_HEAD_FORMAT ) << ": \""
-         << res1.getClipboardHeadFormat() << "\"" << endl;
+         << XxUtil::escapeChars( res1.getClipboardHeadFormat() ) << "\"" << endl;
    }
    if ( res1.getClipboardLineFormat() !=
         res2.getClipboardLineFormat() ) {
       os << searchTokenName( STPARAM(kwdList), CLIPBOARD_LINE_FORMAT ) << ": \""
-         << res1.getClipboardLineFormat() << "\"" << endl;
+         << XxUtil::escapeChars( res1.getClipboardLineFormat() ) << "\"" << endl;
    }
 
    if ( res1.getHordiffType() != res2.getHordiffType() ) {
@@ -1379,9 +1382,9 @@ void XxResParser::listResources( QTextStream& os )
    }
 
    os << searchTokenName( STPARAM(kwdList), CLIPBOARD_HEAD_FORMAT ) << ": \""
-      << res.getClipboardHeadFormat() << "\"" << endl;
+      << XxUtil::escapeChars( res.getClipboardHeadFormat() ) << "\"" << endl;
    os << searchTokenName( STPARAM(kwdList), CLIPBOARD_LINE_FORMAT ) << ": \""
-      << res.getClipboardLineFormat() << "\"" << endl;
+      << XxUtil::escapeChars( res.getClipboardLineFormat() ) << "\"" << endl;
 
    const char* hdtstr[3] = { "None", "Single", "Multiple" };
    os << searchTokenName( STPARAM(kwdList), HORDIFF_TYPE ) << ": "
@@ -1823,6 +1826,15 @@ QString XxResParser::getKwdName( int kwd )
    return QString( stoken );
 }
 
+//------------------------------------------------------------------------------
+//
+QString XxResParser::getShowOptName( XxShowOpt bo )
+{
+   const char* stoken =
+      searchTokenName( STPARAM(showList), int(bo) );
+   return getKwdName( SHOW ) + "." + QString( stoken );
+}
+
 //------------------------------------------------------------------------------
 //
 QString XxResParser::getBoolOptName( XxBoolOpt bo )
