@@ -87,6 +87,7 @@ enum XxBoolOpt {
    BOOL_DIRDIFF_BUILD_FROM_OUTPUT,
    BOOL_DIRDIFF_RECURSIVE,
    BOOL_USE_INTERNAL_DIFF,
+   BOOL_NULL_HORIZONTAL_MARKERS,
    BOOL_LAST // Not a real resource.
 };
 
@@ -120,6 +121,7 @@ enum XxAccel {
    ACCEL_EDIT_LEFT,
    ACCEL_EDIT_MIDDLE,
    ACCEL_EDIT_RIGHT,
+   ACCEL_SAVE_OPTIONS,
    ACCEL_SEARCH,
    ACCEL_SEARCH_FORWARD,
    ACCEL_SEARCH_BACKWARD,
@@ -187,6 +189,7 @@ enum XxAccel {
    ACCEL_TOGGLE_LINE_NUMBERS,
    ACCEL_TOGGLE_MARKERS,
    ACCEL_TOGGLE_VERTICAL_LINE,
+   ACCEL_TOGGLE_NULL_HORIZONTAL_MARKERS,
    ACCEL_TOGGLE_OVERVIEW,
    ACCEL_TOGGLE_SHOW_FILENAMES,
    ACCEL_HORDIFF_NONE,
@@ -201,9 +204,10 @@ enum XxAccel {
    ACCEL_IGNORE_FILE_RIGHT,
    ACCEL_HELP_MAN_PAGE,
    ACCEL_HELP_ON_CONTEXT,
-   ACCEL_HELP_GEN_INIT_FILE,
    ACCEL_HELP_ABOUT,
    ACCEL_MERGED_CLOSE,
+   ACCEL_FONT_RESIZE_BIGGER,
+   ACCEL_FONT_RESIZE_SMALLER,
    ACCEL_LAST // Not a real resource.
 };
 
