@@ -90,7 +90,7 @@ enum SkipType { SK_NOSKIP = 0, SK_UNSEL = 1, SK_NEITHER = 2, SK_EMPTY = 3 };
 
 bool flag = 0;
 
-#ifdef XX_DEBUG
+#ifdef XX_DEBUG_TEXT
 //------------------------------------------------------------------------------
 //
 void traceFontMetrics( const QFontMetrics& fm )
@@ -122,35 +122,45 @@ inline void rentxt(
    int&                xpx,
    const int           wwidth,
    const int           y,
-   const QFontMetrics& fm
+   const QFontMetrics& fm,
+   const bool          renNulHorizMarkers
 )
 {
    QRect brect;
+
+   if ( renNulHorizMarkers && xch == xend ) {
+      p.fillRect(
+         XX_RED_RECT( xpx - 1, y, 1, fm.lineSpacing() ),
+         p.backgroundColor()
+      );
+      return;
+   }
+
    while ( xch < xend ) {
       int rlen = xch_draw_delta;
       if ( (xch + rlen) > xend ) {
          rlen = xend - xch;
       }
-      
+
       QString str;
       XX_CHECK( rlen > 0 ); // always true, because xch < xend
       str.setLatin1( renderedText + xch, rlen );
       int nw = fm.width( str, rlen );
 
 #ifndef XX_DRAWTEXT_DRAWS_BACKGROUND
-      p.eraseRect( 
+      p.eraseRect(
          XX_RED_RECT( xpx, y, nw, fm.lineSpacing() )
       );
 #endif
 
       p.drawText(
          XX_RED_RECT( xpx, y, wwidth - xpx, fm.lineSpacing() ),
-         Qt::AlignLeft | Qt::AlignTop, 
+         Qt::AlignLeft | Qt::AlignTop,
          str, rlen,
          &brect
       );
 
-#ifdef XX_DEBUG
+#ifdef XX_DEBUG_TEXT
       // For debug only, could be removed.
       if ( flag ) {
          QPen pushPen = p.pen();
@@ -163,10 +173,10 @@ inline void rentxt(
          p.setPen( pushPen );
       }
 #endif
-      
+
       xpx += XX_RED_WIDTH( nw ); // XX_RED_WIDTH( brect.width() );
       xch += rlen;
-      
+
       if ( xpx > wwidth ) {
          break;
       }
@@ -187,12 +197,12 @@ XX_NAMESPACE_BEGIN
 
 //------------------------------------------------------------------------------
 //
-XxText::XxText( 
-   XxApp*        app, 
+XxText::XxText(
+   XxApp*        app,
    XxScrollView* sv,
-   const XxFno   no, 
-   QWidget*      parent, 
-   const char*   name 
+   const XxFno   no,
+   QWidget*      parent,
+   const char*   name
 ) :
    QFrame( parent, name, WResizeNoErase ),
    _app( app ),
@@ -293,6 +303,7 @@ void XxText::drawContents( QPainter* pp )
    XxDln cursorLine = _app->getCursorLine();
    uint horizontalPos = _sv->getHorizontalPos();
    uint tabWidth = resources.getTabWidth();
+   bool hideCR = resources.getBoolOpt( BOOL_HIDE_CR );
 
    int selRegY1 = -1;
    int selRegY2 = -1;
@@ -341,6 +352,9 @@ void XxText::drawContents( QPainter* pp )
 
    SkipType skip = SK_NOSKIP;
 
+   bool renNulHorizMarkers =
+      resources.getBoolOpt( BOOL_NULL_HORIZONTAL_MARKERS );
+
    int py = -1;
    XxLine::Type prevtype;
    XxLine::Type type = XxLine::SAME;
@@ -360,7 +374,7 @@ void XxText::drawContents( QPainter* pp )
       XxColor idtype, idtypeSup;
       if ( isMerged() ) {
 
-         if ( type != XxLine::SAME && 
+         if ( type != XxLine::SAME &&
               type != XxLine::DIRECTORIES ) {
 
             XxLine::Selection sel = line.getSelection();
@@ -377,12 +391,16 @@ void XxText::drawContents( QPainter* pp )
                                             bcolor, fcolor );
 
                   p.setBackgroundColor( bcolor );
-                  QBrush brush( fcolor );
-                  brush.setStyle( QBrush::BDiagPattern );
-            
+                  QBrush brush( bcolor );
+
                   p.fillRect( XX_RED_RECT( 0, y, w, HEIGHT_UNSEL_REGION ),
                               brush );
-                  
+
+                  brush.setColor( fcolor );
+                  brush.setStyle( QBrush::BDiagPattern );
+                  p.fillRect( XX_RED_RECT( 0, y, w, HEIGHT_UNSEL_REGION ),
+                              brush );
+
                   py = y;
                   y += HEIGHT_UNSEL_REGION;
                }
@@ -408,7 +426,7 @@ void XxText::drawContents( QPainter* pp )
                   QBrush brush( bcolor );
                   p.fillRect( XX_RED_RECT( 0, y, w, HEIGHT_NEITHER_REGION ),
                               brush );
-                  
+
                   py = y;
                   y += HEIGHT_NEITHER_REGION;
                }
@@ -467,7 +485,7 @@ void XxText::drawContents( QPainter* pp )
 
          uint length;
          const char* lineText = files[renNo]->getTextLine( fline, length );
-         
+
          const int bhd = 0;
          int ehd;
          const char* renderedText;
@@ -485,8 +503,8 @@ void XxText::drawContents( QPainter* pp )
             XX_CHECK( c < hbufferSize );
             hbuffer0[c] = -1;
 
-            renderedText = files[renNo]->renderTextWithTabs( 
-               lineText, length, tabWidth, ehd, hbuffer0
+            renderedText = files[renNo]->renderTextWithTabs(
+               lineText, length, tabWidth, hideCR, ehd, hbuffer0
             );
 
             // Append ehd and end array with -1.  We make sure there was enough
@@ -496,11 +514,11 @@ void XxText::drawContents( QPainter* pp )
             XX_CHECK( c < hbufferSize );
          }
          else {
-            renderedText = files[renNo]->renderTextWithTabs( 
-               lineText, length, tabWidth, ehd, 0
+            renderedText = files[renNo]->renderTextWithTabs(
+               lineText, length, tabWidth, hideCR, ehd, 0
             );
          }
-         
+
          // Loop to find an character reasonably close before the beginning of
          // the visible window in x.
          QString chunk;
@@ -517,8 +535,8 @@ void XxText::drawContents( QPainter* pp )
             chunk.setLatin1( renderedText + xch, rlen );
             // FIXME check somehow that this actually corresponds to the
             // rendered measure.
-            QRect brect = fm.boundingRect( 
-               -128, -128, 8192, 2048, 
+            QRect brect = fm.boundingRect(
+               -128, -128, 8192, 2048,
                Qt::AlignLeft | Qt::AlignTop | Qt::SingleLine,
                chunk, rlen
             );
@@ -568,7 +586,8 @@ void XxText::drawContents( QPainter* pp )
                   p, renderedText,
                   xch, hbuffer0[c],
                   xpx,
-                  w, y, fm
+                  w, y, fm,
+                  renNulHorizMarkers
                );
                if ( xpx > w ) {
                   break;
@@ -585,7 +604,7 @@ void XxText::drawContents( QPainter* pp )
 
             //
             // Render without horizontal diffs.
-            //            
+            //
 
             p.setPen( fcolor );
             p.setBackgroundColor( bcolor );
@@ -593,10 +612,11 @@ void XxText::drawContents( QPainter* pp )
                p, renderedText,
                xch, ehd,
                xpx,
-               w, y, fm
+               w, y, fm,
+               false
             );
          }
-         
+
          // Filler part.
          int fillerwidth = w - xpx;
          if ( fillerwidth > 0 ) {
@@ -630,18 +650,18 @@ void XxText::drawContents( QPainter* pp )
 
             y += fm.lineSpacing();
          }
-         else { 
+         else {
             // For merged view, render the first empty line as thin.
             if ( skip != SK_EMPTY && type != prevtype ) {
                skip = SK_EMPTY;
-               
+
                //
                // Draw empty marker.
                //
                QBrush brush( bcolor );
                p.fillRect( XX_RED_RECT( 0, y, w, HEIGHT_EMPTY_REGION ),
                            brush );
-                  
+
                py = y;
                y += HEIGHT_EMPTY_REGION;
             }
@@ -760,10 +780,10 @@ void XxText::mousePressEvent( QMouseEvent* event )
          _grabMode = REGION_SELECT;
          _grabTopLine = _sv->getTopLine();
          _grabDeltaLineNo = dlineno;
-         
+
          QClipboard* cb = QkApplication::clipboard();
          cb->clear();
-         
+
          _regionSelect[0] = _grabTopLine + dlineno;
          _regionSelect[1] = -1;
          update();
@@ -772,7 +792,7 @@ void XxText::mousePressEvent( QMouseEvent* event )
    }
 
    // Interactive toggling of debug drawing structures.
-#ifdef XX_DEBUG
+#ifdef XX_DEBUG_TEXT
    if ( event->button() == MidButton ) {
       if ( event->state() & ControlButton ) {
          flag = !flag;
@@ -805,15 +825,15 @@ void XxText::mousePressEvent( QMouseEvent* event )
          // Delete line.
          diffs->selectLine( lineno, XxLine::NEITHER );
       }
-      else { 
+      else {
          // Select appropriate side.
          diffs->selectLine( lineno, XxLine::Selection( _no ) );
       }
 
       // Compute line text.
       const XxLine& line = diffs->getLine( lineno );
-      if ( line.getType() == XxLine::SAME || 
-           line.getType() == XxLine::DIRECTORIES || 
+      if ( line.getType() == XxLine::SAME ||
+           line.getType() == XxLine::DIRECTORIES ||
            line.getSelection() == XxLine::Selection( _no ) ) {
          XxFln fline = line.getLineNo( _no );
          if ( fline != -1 ) {
@@ -826,7 +846,7 @@ void XxText::mousePressEvent( QMouseEvent* event )
                }
                if ( resources.getBoolOpt( BOOL_FORMAT_CLIPBOARD_TEXT )
                     == true ) {
-                  QString forline = formatClipboard( 
+                  QString forline = formatClipboard(
                      clipboardLineFormat, _no, fline, filename, adt
                   );
                   textCopy += forline;
@@ -865,7 +885,7 @@ void XxText::mousePressEvent( QMouseEvent* event )
    QClipboard* cb = QkApplication::clipboard();
    cb->setText( textCopy );
 
-   if ( event->button() == LeftButton || 
+   if ( event->button() == LeftButton ||
         event->button() == MidButton ) {
       _app->setCursorLine( lineno );
    }
@@ -896,7 +916,7 @@ QString XxText::getRegionText( XxDln start, XxDln end ) const
       if ( fline != -1 ) {
 
          if ( ! headerAdded ) {
-            QString header = formatClipboard( 
+            QString header = formatClipboard(
                clipboardHeadFormat, _no, fline, filename, QString("")
             );
             textCopy += header;
@@ -912,7 +932,7 @@ QString XxText::getRegionText( XxDln start, XxDln end ) const
             }
             if ( resources.getBoolOpt( BOOL_FORMAT_CLIPBOARD_TEXT )
                  == true ) {
-               QString forline = formatClipboard( 
+               QString forline = formatClipboard(
                   clipboardLineFormat, _no, fline, filename, adt
                );
                textCopy += forline;
@@ -965,9 +985,9 @@ void XxText::mouseMoveEvent( QMouseEvent* event )
             _regionSelect[0] = newSel;
          }
          // Keep em sorted.
-         XX_CHECK( ( _regionSelect[0] == -1 || _regionSelect[1] == -1 ) || 
+         XX_CHECK( ( _regionSelect[0] == -1 || _regionSelect[1] == -1 ) ||
                    ( _regionSelect[0] <= _regionSelect[1] ) );
-         
+
          update();
 
          // Compute region text.
@@ -1019,6 +1039,13 @@ void XxText::mouseDoubleClickEvent( QMouseEvent* event )
    }
 }
 
+//------------------------------------------------------------------------------
+//
+void XxText::wheelEvent( QWheelEvent* e )
+{
+   QApplication::sendEvent( _sv, e );
+}
+
 //------------------------------------------------------------------------------
 //
 void XxText::resizeEvent( QResizeEvent* )
@@ -1055,65 +1082,6 @@ uint XxText::computeMergedLines() const
    XxLine::Type prevtype;
    XxLine::Type type = XxLine::SAME;
    for ( XxDln icurline = 1; icurline <= diffs->getNbLines(); ++icurline ) {
-// #if 0 
-
-//       prevtype = type;
-//       const XxLine& line = diffs->getLine( icurline++ );
-//       type = line.getType();
-
-//       XxFno renNo = _no;
-//       if ( isMerged() ) {
-
-//          if ( type != XxLine::SAME && 
-//               type != XxLine::DIRECTORIES ) {
-
-//             XxLine::Selection sel = line.getSelection();
-//             if ( sel == XxLine::UNSELECTED ) {
-//                if ( skip != SK_UNSEL ) {
-//                   skip = SK_UNSEL;
-//                   y += HEIGHT_UNSEL_REGION;
-//                }
-//                continue;
-//             }
-//             else if ( sel == XxLine::NEITHER ) {
-//                if ( skip != SK_NEITHER ) {
-//                   skip = SK_NEITHER;
-//                   y += HEIGHT_NEITHER_REGION;
-//                }
-//                continue;
-//             }
-//             else {
-//                // Render selected side.
-//                renNo = int(sel);
-//                skip = SK_NOSKIP;
-//             }
-//          }
-//          else {
-//             renNo = 0; // Any side would be fine for SAME regions.
-//             skip = SK_NOSKIP;
-//          }
-//       }
-
-//       // Render text.
-//       XxFln fline = line.getLineNo( renNo );
-//       if ( fline != -1 ) {
-//          y += fm.lineSpacing();
-//       }
-//       else {
-
-//          if ( !isMerged() ) {
-//             y += fm.lineSpacing();
-//          }
-//          else { 
-//             // For merged view, render the first empty line as thin.
-//             if ( skip != SK_EMPTY && type != prevtype ) {
-//                skip = SK_EMPTY;
-//                y += HEIGHT_EMPTY_REGION;
-//             }
-//          }
-//       }
-//    }
-// #endif
 
       // Get line to display.
       prevtype = type;
@@ -1123,7 +1091,7 @@ uint XxText::computeMergedLines() const
       XxFno renNo = _no;
       if ( isMerged() ) {
 
-         if ( type != XxLine::SAME && 
+         if ( type != XxLine::SAME &&
               type != XxLine::DIRECTORIES ) {
 
             XxLine::Selection sel = line.getSelection();
@@ -1179,11 +1147,11 @@ uint XxText::computeMergedLines() const
             y += fm.lineSpacing();
             XX_LOCAL_TRACE( "fm.lineSpacing() " << y );
          }
-         else { 
+         else {
             // For merged view, render the first empty line as thin.
             if ( skip != SK_EMPTY && type != prevtype ) {
                skip = SK_EMPTY;
-               
+
                //
                // Draw empty marker.
                //
@@ -1195,7 +1163,7 @@ uint XxText::computeMergedLines() const
    }
 
 
-   XX_LOCAL_TRACE( "y = " << y 
+   XX_LOCAL_TRACE( "y = " << y
              << " ls = " << fm.lineSpacing()
              << " difflines = " << diffs->getNbLines() );
 
@@ -1215,7 +1183,7 @@ QString XxText::formatClipboard(
 )
 {
    QString forline = clipboardLineFormat;
-   
+
    typedef int PosType;
    PosType notfound = -1;
 
