@@ -736,7 +736,7 @@ void XxUtil::spawnCommand(
    }
    errorf = stderr;
 
-#if 0
+#if KEPT_FOR_HISTORY
    /*
     * Read pipe until end of file. End of file indicates that outputf closed its
     * standard out (probably meaning it terminated).
@@ -1013,4 +1013,29 @@ unsigned int XxUtil::toTime_t(
 #endif
 }
 
+//------------------------------------------------------------------------------
+//
+QString XxUtil::escapeChars( const QString& format )
+{
+   QString newFormat = format;
+
+   newFormat.replace( "\n", "\\n" );
+   newFormat.replace( "\r", "\\r" );
+   newFormat.replace( "\"", "\\\"" );
+
+   return newFormat;
+}
+
+//------------------------------------------------------------------------------
+//
+QString XxUtil::unescapeChars( const QString& format )
+{
+   QString newFormat = format;
+
+   newFormat.replace( "\\n", "\n" );
+   newFormat.replace( "\\r", "\r" );
+   newFormat.replace( "\\\"", "\"" );
+   return newFormat;
+}
+
 XX_NAMESPACE_END
